#![feature(iter_intersperse)]
#![feature(trait_alias)]
use std::io::{BufRead, Write};
use command::{Command, Env, Args};

mod command;

pub static SPECIAL_CHAR: char = '$';

fn main() {
    let stdin = std::io::stdin();
    let mut in_lock = stdin.lock();

    let stdout = std::io::stdout();
    let mut out_lock = stdout.lock();
    
    let mut env= Env::new();
    env.set("HOME", "/a/fake/dir");
    env.set("?", "0");

    let commands = &Command::all_commands(&env);
    println!("Loaded {} commands", commands.len());

    loop {
        let mut buf = String::new();

        let default_ps1 = "> ".to_string();
        let ps1 = env.get("PS1").unwrap_or(&default_ps1).as_bytes();
        out_lock.write(ps1).unwrap();
        out_lock.flush().unwrap();

        in_lock
            .read_line(&mut buf)
            .expect("Failed to read in to buffer.");

        // TODO split at ' ' but don't split when it's surrounded by quotes '"'
        let args= &buf.trim().split(' ').map(|f| f.to_owned()).collect::<Vec<String>>();
        let args = env.apply_args_substitution(args);
        let mut args: Args = &args;

        // let mut args: Args = &buf.trim().split(' ').collect::<Vec<&str>>();
        let mut options = commands;

        loop {
            match args.first() {
                Some(cmd_arg) => {
                    // Try to find a subcommand to run
                    match Command::choose(cmd_arg, options) {
                        Some(choice) => {
                            // No sub commands left to choose from, just pass the input
                            if choice.sub_commands.is_empty() {
                                choice.call(&args, &mut env);
                                break;
                            } 

                            args = &args[1..];
                            options = &choice.sub_commands;

                            // a sort of last chance before running into hella edgecases
                            if args.is_empty()
                            || Command::choose(&args[0], options).is_none() // yes, this means that we search for a choice twice if it misses, but the search takes all of 0ms, so it's fine.
                            {
                                choice.call(&args, &mut env);
                                break;
                            }
                        }
                        // When no good choice presents it's self
                        None => {
                            // perhaps they are trying to use builtins?
                            parse_lang(&args, &mut env);
                            break;
                        },
                    }
                }
                None => unimplemented!("Couldn't get index 0 of the args"),
            }
        }
    }
}

fn parse_lang(tokens: Args, env: &mut Env) {
    if let Some(first) =  tokens.first() {
        if let Some((var, val)) = first.split_once('=') {
            let var = var.replace(SPECIAL_CHAR, "");

            // Make it so that strings don't get spliced up
            let val = if val.starts_with('"') {
                let mut buf = String::new();
                buf += &val;
                for i in &tokens[1..] {
                    buf += " ";
                    buf += i;
                    if i.ends_with('"') {
                        break;
                    }
                }
                buf.replace('"', "")
            } else {
                val.to_owned()
            };
            env.set(&var, &val);
        } else {
            println!("first token is not a command but also doesn't include '='");
        }
    }
}
